# -*- coding: utf-8 -*-
import datetime
import filecmp
from decimal import Decimal
from pathlib import Path

import pytest
from dateutil import tz

from xero_python.accounting import (
    AccountingApi,
    models,
    Account,
    Accounts,
    AccountType,
    CurrencyCode,
    LineAmountTypes,
    Invoices,
    Invoice,
    Contact,
    Payment,
    HistoryRecords,
    RequestEmpty,
    HistoryRecord,
    Address,
    ContactGroup,
    ContactPerson,
    Phone,
    LineItem,
    Attachments,
    Attachment,
)
from xero_python.api_client import ApiClient
from xero_python.rest import RESTClientObject


@pytest.fixture
def accounting_api(api_client):
    return AccountingApi(api_client=api_client)


@pytest.fixture
def sandbox_accounting_api(api_client):
    return AccountingApi(
        api_client=api_client,
        base_url="https://xero-accounting.getsandbox.com/api.xro/2.0",
    )


@pytest.fixture
def invoice_pdf():
    return Path(__file__).resolve().parent.joinpath("fixtures/inv-0032.pdf")


def test_accounting_api_instance_configuration(accounting_api):
    """
    Test AccountingApi() configured correctly
    """
    assert isinstance(accounting_api, AccountingApi)
    assert isinstance(accounting_api.api_client, ApiClient)
    assert isinstance(accounting_api.api_client.rest_client, RESTClientObject)


@pytest.mark.vcr()
def test_get_organisations(accounting_api: AccountingApi, xero_tenant_id):
    """
    Test AccountingApi.get_organisations()
    """
    # Given correct api client and xero tenant id
    # When fetching organisations
    response = accounting_api.get_organisations(xero_tenant_id=xero_tenant_id)
    # Then expect correct response with organisations tenant has access to
    assert isinstance(response, models.Organisations)
    organisations = response.organisations
    assert isinstance(organisations, list)
    assert len(organisations) == 1
    org = organisations[0]
    assert isinstance(org, models.Organisation)
    assert org.organisation_id
    assert org.is_demo_company


@pytest.mark.vcr()
def test_invoice_attachment_upload_and_download(
    accounting_api, xero_tenant_id, invoice_pdf
):
    """This is a full integration test, tested steps are:

       1. get first page of invoices
       2. choose first invoice
       3. upload test pdf file as invoice attachment
       4. confirm the invoice has new attachment
       5. download uploaded pdf file
    """

    # 1. get first page of invoices
    invoices = accounting_api.get_invoices(xero_tenant_id, page=1)
    assert isinstance(invoices, Invoices)
    assert len(invoices.invoices)
    # 2. choose first invoice
    invoice = invoices.invoices[0]
    assert isinstance(invoice, Invoice)
    # 3. upload test pdf file as invoice attachment
    include_online = True
    with invoice_pdf.open("rb") as pdf:
        response = accounting_api.create_invoice_attachment_by_file_name(
            xero_tenant_id,
            invoice.invoice_id,
            file_name=invoice_pdf.name,
            include_online=include_online,
            body=pdf.read(),
        )
    assert isinstance(response, Attachments)
    assert len(response.attachments) == 1
    attachment = response.attachments[0]
    assert isinstance(attachment, Attachment)
    assert attachment.file_name == invoice_pdf.name
    assert attachment.include_online == include_online
    assert attachment.content_length == invoice_pdf.stat().st_size
    # 4. confirm the invoice has new attachment
    attachments = accounting_api.get_invoice_attachments(
        xero_tenant_id, invoice.invoice_id
    )
    assert isinstance(attachments, Attachments)
    assert len(attachments.attachments)
    for invoice_attachment in attachments.attachments:
        if invoice_attachment.attachment_id == attachment.attachment_id:
            assert invoice_attachment == attachment
            break
    else:
        raise AssertionError("uploaded invoice attachment not found")
    # 5. download uploaded pdf file
    temp_pdf_path = accounting_api.get_invoice_attachment_by_id(
        xero_tenant_id,
        invoice.invoice_id,
        attachment.attachment_id,
        content_type=attachment.mime_type,
    )
    assert isinstance(temp_pdf_path, str)
    assert temp_pdf_path
    temp_pdf = Path(temp_pdf_path)
    assert temp_pdf.exists()
    assert filecmp.cmp(str(invoice_pdf), temp_pdf_path, shallow=False)
    # test cleanup
    temp_pdf.unlink()


@pytest.mark.sandbox
def test_get_accounts(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test data
    expected = Accounts(
        accounts=[
            Account(
                account_id="ebd06280-af70-4bed-97c6-7451a454ad85",
                bank_account_number="0209087654321050",
                bank_account_type="BANK",
                code="091",
                currency_code=CurrencyCode.NZD,
                enable_payments_to_account=False,
                has_attachments=False,
                name="Business Savings Account",
                tax_type="NONE",
                type=AccountType.BANK,
            ),
            Account(
                account_id="7d05a53d-613d-4eb2-a2fc-dcb6adb80b80",
                code="200",
                description="Income from any normal business activity",
                enable_payments_to_account=False,
                has_attachments=False,
                name="Sales",
                tax_type="OUTPUT2",
                type=AccountType.REVENUE,
            ),
        ]
    )
    # When getting all accounts
    result = sandbox_accounting_api.get_accounts(xero_tenant_id)
    # Then expect all accounts to be received
    assert result == expected


@pytest.mark.sandbox
def test_get_account(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test data
    account_id = "99ce6032-0678-4aa0-8148-240c75fee33a"
    expected = Accounts(
        accounts=[
            Account(
                _class="EXPENSE",
                account_id=account_id,
                code="123456",
                description="Hello World",
                enable_payments_to_account=False,
                has_attachments=False,
                name="FooBar",
                reporting_code="EXP",
                reporting_code_name="Expense",
                show_in_expense_claims=False,
                status="ACTIVE",
                tax_type="INPUT",
                type=AccountType.EXPENSE,
                updated_date_utc=datetime.datetime(
                    2019, 2, 22, 1, 2, 39, 120000, tzinfo=tz.UTC
                ),
            )
        ]
    )
    # When getting account
    result = sandbox_accounting_api.get_account(xero_tenant_id, account_id)
    # Then expect one account to be received
    assert result == expected


@pytest.mark.sandbox
def test_create_account(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test account data
    expected = Account(
        _class="EXPENSE",
        account_id="66b262e2-561e-423e-8937-47d558f13442",
        code="123456",
        description="Hello World",
        enable_payments_to_account=False,
        has_attachments=False,
        name="Foobar",
        reporting_code="EXP",
        reporting_code_name="Expense",
        show_in_expense_claims=False,
        status="ACTIVE",
        tax_type="INPUT",
        type=AccountType.EXPENSE,
        updated_date_utc=datetime.datetime(
            2019, 2, 21, 23, 59, 9, 320000, tzinfo=tz.UTC
        ),
    )
    account = Account(
        code=expected.code,
        name=expected.name,
        type=expected.type,
        bank_account_number=None,  # required but not set in sandbox version
    )
    # When creating account
    result = sandbox_accounting_api.create_account(xero_tenant_id, account)
    # Then expect new account to be created
    assert isinstance(result, Accounts)
    assert len(result.accounts) == 1
    result_account = result.accounts[0]
    assert result_account == expected


@pytest.mark.sandbox
def test_update_account(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test data
    account_id = "99ce6032-0678-4aa0-8148-240c75fee33a"
    expected = accounts = Accounts(
        accounts=[
            Account(
                _class="EXPENSE",
                account_id=account_id,
                code="654321",
                description="Good Bye World",
                enable_payments_to_account=False,
                has_attachments=False,
                name="BarFoo",
                reporting_code="EXP",
                reporting_code_name="Expense",
                show_in_expense_claims=False,
                status="ACTIVE",
                tax_type="INPUT",
                type=AccountType.EXPENSE,
                updated_date_utc=datetime.datetime(
                    2019, 2, 22, 0, 29, 49, 333000, tzinfo=tz.UTC
                ),
            )
        ]
    )
    # When updating account
    result = sandbox_accounting_api.update_account(xero_tenant_id, account_id, accounts)
    # Then expect the account to be updated
    assert result == expected


@pytest.mark.sandbox
def test_delete_account(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test data
    account_id = "7f3c0bec-f3e7-4073-b4d6-cc56dd027ef1"
    expected = Accounts(
        accounts=[
            Account(
                _class="EXPENSE",
                account_id=account_id,
                code="123456",
                description="Hello World",
                enable_payments_to_account=False,
                has_attachments=False,
                name="FooBar",
                reporting_code="EXP",
                reporting_code_name="Expense",
                show_in_expense_claims=False,
                status="DELETED",
                tax_type="INPUT",
                type=AccountType.EXPENSE,
                updated_date_utc=datetime.datetime(
                    2019, 2, 22, 1, 16, 57, 210000, tzinfo=tz.UTC
                ),
            )
        ]
    )
    # When deleting account
    result = sandbox_accounting_api.delete_account(xero_tenant_id, account_id)
    # Then expect the account to be deleted
    assert result == expected


@pytest.mark.sandbox
def test_get_account_attachments(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test data
    account_id = "7f3c0bec-f3e7-4073-b4d6-cc56dd027ef1"
    expected = Attachments(
        attachments=[
            Attachment(
                attachment_id="52a643be-cd5c-489f-9778-53a9fd337756",
                content_length=Decimal("2878711"),
                file_name="sample5.jpg",
                include_online=None,
                mime_type="image/jpg",
                url="https://api.xero.com/api.xro/2.0/Accounts/"
                "da962997-a8bd-4dff-9616-01cdc199283f/Attachments/sample5.jpg",
            )
        ]
    )
    # When getting account attachments
    result = sandbox_accounting_api.get_account_attachments(xero_tenant_id, account_id)
    # Then expect the account attachments to be received
    assert result == expected


@pytest.mark.sandbox
def test_get_invoices(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test invoices data
    # When getting all invoices
    result = sandbox_accounting_api.get_invoices(xero_tenant_id)
    # Then expect correct invoices received
    expected = Invoices(
        invoices=[
            Invoice(
                amount_credited=Decimal("0.00"),
                amount_due=Decimal("0.00"),
                amount_paid=Decimal("0.00"),
                contact=Contact(
                    contact_id="a3675fc4-f8dd-4f03-ba5b-f1870566bcd7",
                    has_attachments=False,
                    has_validation_errors=False,
                    name="Barney Rubble-83203",
                    addresses=[],
                    contact_groups=[],
                    contact_persons=[],
                    phones=[],
                ),
                credit_notes=[],
                line_items=[],
                overpayments=[],
                payments=[],
                prepayments=[],
                currency_code=CurrencyCode.NZD,
                currency_rate=Decimal("1.000000"),
                date=datetime.date(2018, 10, 20),
                due_date=datetime.date(2018, 12, 30),
                has_attachments=False,
                has_errors=False,
                invoice_id="d4956132-ed94-4dd7-9eaa-aa22dfdf06f2",
                invoice_number="INV-0001",
                is_discounted=False,
                line_amount_types=LineAmountTypes.EXCLUSIVE,
                reference="Red Fish, Blue Fish",
                sent_to_contact=True,
                status="VOIDED",
                sub_total=Decimal("40.00"),
                total=Decimal("40.00"),
                total_tax=Decimal("0.00"),
                type="ACCREC",
                updated_date_utc=datetime.datetime(
                    2018, 11, 2, 16, 31, 30, 160000, tzinfo=tz.UTC
                ),
            ),
            Invoice(
                amount_credited=Decimal("0.00"),
                amount_due=Decimal("0.00"),
                amount_paid=Decimal("46.00"),
                contact=Contact(
                    contact_id="a3675fc4-f8dd-4f03-ba5b-f1870566bcd7",
                    has_attachments=False,
                    has_validation_errors=False,
                    name="Barney Rubble-83203",
                    addresses=[],
                    contact_groups=[],
                    contact_persons=[],
                    phones=[],
                ),
                credit_notes=[],
                line_items=[],
                overpayments=[],
                prepayments=[],
                currency_code=CurrencyCode.NZD,
                currency_rate=Decimal("1.000000"),
                date=datetime.date(2018, 10, 20),
                due_date=datetime.date(2018, 12, 30),
                fully_paid_on_date=datetime.date(2018, 11, 29),
                has_attachments=False,
                has_errors=False,
                invoice_id="046d8a6d-1ae1-4b4d-9340-5601bdf41b87",
                invoice_number="INV-0002",
                is_discounted=False,
                line_amount_types=LineAmountTypes.EXCLUSIVE,
                payments=[
                    Payment(
                        amount=Decimal("46.00"),
                        currency_rate=Decimal("1.000000"),
                        date=datetime.date(2018, 11, 29),
                        has_account=False,
                        has_validation_errors=False,
                        payment_id="99ea7f6b-c513-4066-bc27-b7c65dcd76c2",
                    )
                ],
                reference="Red Fish, Blue Fish",
                sent_to_contact=True,
                status="PAID",
                sub_total=Decimal("40.00"),
                total=Decimal("46.00"),
                total_tax=Decimal("6.00"),
                type="ACCREC",
                updated_date_utc=datetime.datetime(
                    2018, 11, 2, 16, 36, 32, 690000, tzinfo=tz.UTC
                ),
            ),
            Invoice(
                amount_credited=Decimal("0.00"),
                amount_due=Decimal("115.00"),
                amount_paid=Decimal("0.00"),
                contact=Contact(
                    contact_id="a3675fc4-f8dd-4f03-ba5b-f1870566bcd7",
                    has_attachments=False,
                    has_validation_errors=False,
                    name="Barney Rubble-83203",
                    addresses=[],
                    contact_groups=[],
                    contact_persons=[],
                    phones=[],
                ),
                credit_notes=[],
                line_items=[],
                overpayments=[],
                payments=[],
                prepayments=[],
                currency_code=CurrencyCode.NZD,
                currency_rate=Decimal("1.000000"),
                date=datetime.date(2018, 11, 2),
                due_date=datetime.date(2018, 11, 7),
                has_attachments=False,
                has_errors=False,
                invoice_id="7ef31b20-de17-4312-8382-412f869b1510",
                invoice_number="INV-0003",
                is_discounted=False,
                line_amount_types=LineAmountTypes.EXCLUSIVE,
                reference="",
                status="AUTHORISED",
                sub_total=Decimal("100.00"),
                total=Decimal("115.00"),
                total_tax=Decimal("15.00"),
                type="ACCREC",
                updated_date_utc=datetime.datetime(
                    2018, 11, 2, 16, 37, 28, 927000, tzinfo=tz.UTC
                ),
            ),
        ]
    )
    assert result == expected


@pytest.mark.sandbox
def test_get_invoice_history(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test invoice data
    invoice_id = "d4956132-ed94-4dd7-9eaa-aa22dfdf06f2"
    # When getting invoice history
    result = sandbox_accounting_api.get_invoice_history(xero_tenant_id, invoice_id)
    # Then expect invoice history to be received
    expected = HistoryRecords()  # todo confirm empty response from sandbox is correct
    assert result == expected


@pytest.mark.sandbox
def test_update_or_create_invoices(
    sandbox_accounting_api: AccountingApi, xero_tenant_id
):
    # Given sandbox API, tenant id, and hardcoded test invoices data
    expected = invoices = Invoices(
        invoices=[
            Invoice(
                amount_due=Decimal("40.00"),
                amount_paid=Decimal("0.00"),
                contact=Contact(
                    addresses=[
                        Address(
                            address_type="STREET",
                            attention_to="",
                            city="",
                            country="",
                            postal_code="",
                            region="",
                        ),
                        Address(
                            address_type="POBOX",
                            attention_to="",
                            city="Anytown",
                            country="USA",
                            postal_code="10101",
                            region="NY",
                        ),
                    ],
                    bank_account_details="",
                    contact_groups=[
                        ContactGroup(
                            contact_group_id="17b44ed7-4389-4162-91cb-3dd5766e4e22",
                            contacts=[],
                            name="Oasis",
                            status="ACTIVE",
                        )
                    ],
                    contact_id="430fa14a-f945-44d3-9f97-5df5e28441b8",
                    contact_persons=[
                        ContactPerson(
                            email_address="debbie@rockstar.com",
                            first_name="Debbie",
                            include_in_emails=False,
                            last_name="Gwyther",
                        )
                    ],
                    contact_status="ACTIVE",
                    email_address="liam@rockstar.com",
                    first_name="Liam",
                    has_attachments=False,
                    has_validation_errors=False,
                    is_customer=True,
                    is_supplier=True,
                    last_name="Gallagher",
                    name="Liam Gallagher",
                    phones=[
                        Phone(
                            phone_area_code="212",
                            phone_country_code="",
                            phone_number="222-2222",
                            phone_type="DEFAULT",
                        ),
                        Phone(
                            phone_area_code="",
                            phone_country_code="",
                            phone_number="",
                            phone_type="DDI",
                        ),
                        Phone(
                            phone_area_code="212",
                            phone_country_code="",
                            phone_number="333-2233",
                            phone_type="FAX",
                        ),
                        Phone(
                            phone_area_code="212",
                            phone_country_code="",
                            phone_number="444-3433",
                            phone_type="MOBILE",
                        ),
                    ],
                    purchases_tracking_categories=[],
                    sales_tracking_categories=[],
                    updated_date_utc=datetime.datetime(
                        2019, 3, 5, 0, 54, 41, 53000, tzinfo=tz.UTC
                    ),
                ),
                currency_code=CurrencyCode.NZD,
                currency_rate=Decimal("1.000000"),
                date=datetime.date(2019, 3, 11),
                due_date=datetime.date(2018, 12, 10),
                has_attachments=False,
                has_errors=False,
                invoice_id="ed255415-e141-4150-aab7-89c3bbbb851c",
                invoice_number="INV-0007",
                is_discounted=False,
                line_amount_types=LineAmountTypes.EXCLUSIVE,
                line_items=[
                    LineItem(
                        account_code="200",
                        description="Acme Tires",
                        line_amount=Decimal("40.00"),
                        line_item_id="5f7a612b-fdcc-4d33-90fa-a9f6bc6db32f",
                        quantity=Decimal("2.0000"),
                        tax_amount=Decimal("0.00"),
                        tax_type="NONE",
                        tracking=[],
                        unit_amount=Decimal("20.00"),
                    )
                ],
                overpayments=[],
                prepayments=[],
                reference="Website Design",
                sent_to_contact=False,
                status="AUTHORISED",
                status_attribute_string="OK",
                sub_total=Decimal("40.00"),
                total=Decimal("40.00"),
                total_tax=Decimal("0.00"),
                type="ACCREC",
                updated_date_utc=datetime.datetime(
                    2019, 3, 11, 17, 58, 46, 117000, tzinfo=tz.UTC
                ),
            )
        ]
    )
    # When updating or creating invoices
    result = sandbox_accounting_api.update_or_create_invoices(xero_tenant_id, invoices)
    # Then expect invoices to be created or updated
    assert result == expected


@pytest.mark.sandbox
def test_create_invoice_history(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test data
    invoice_id = "d4956132-ed94-4dd7-9eaa-aa22dfdf06f2"
    history_records = HistoryRecords(
        [
            HistoryRecord(
                details="Received through the Xero API from ABC Org",
                changes="Approved",
                user="System Generated",
                date_utc=datetime.datetime(2018, 2, 28, 21, 2, 11, tzinfo=tz.UTC),
            ),
            HistoryRecord(
                details="INV-0041 to ABC Furniture for 100.00.",
                changes="Edited",
                user="Mac Haag",
                date_utc=datetime.datetime(2018, 2, 28, 21, 1, 29, tzinfo=tz.UTC),
            ),
        ]
    )
    # When creating invoice history
    result = sandbox_accounting_api.create_invoice_history(
        xero_tenant_id, invoice_id, history_records
    )
    # Then expect created invoice history records
    expected = HistoryRecords()  # todo confirm empty response from sandbox is correct
    assert result == expected


@pytest.mark.sandbox
def test_email_invoice(sandbox_accounting_api: AccountingApi, xero_tenant_id):
    # Given sandbox API, tenant id, and hardcoded test data
    invoice_id = "d4956132-ed94-4dd7-9eaa-aa22dfdf06f2"
    request_empty = RequestEmpty()
    # When email invoice
    result = sandbox_accounting_api.email_invoice(
        xero_tenant_id, invoice_id, request_empty
    )
    # Then expect email invoiced
    assert result is None, "email_invoice doesn't return any data"



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class PayRunCalendar(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "payroll_calendar_id": "str",
        "name": "str",
        "calendar_type": "str",
        "period_start_date": "date",
        "period_end_date": "date",
        "payment_date": "date",
        "updated_date_utc": "datetime",
    }

    attribute_map = {
        "payroll_calendar_id": "payrollCalendarID",
        "name": "name",
        "calendar_type": "calendarType",
        "period_start_date": "periodStartDate",
        "period_end_date": "periodEndDate",
        "payment_date": "paymentDate",
        "updated_date_utc": "updatedDateUTC",
    }

    def __init__(
        self,
        payroll_calendar_id=None,
        name=None,
        calendar_type=None,
        period_start_date=None,
        period_end_date=None,
        payment_date=None,
        updated_date_utc=None,
    ):  # noqa: E501
        """PayRunCalendar - a model defined in OpenAPI"""  # noqa: E501

        self._payroll_calendar_id = None
        self._name = None
        self._calendar_type = None
        self._period_start_date = None
        self._period_end_date = None
        self._payment_date = None
        self._updated_date_utc = None
        self.discriminator = None

        if payroll_calendar_id is not None:
            self.payroll_calendar_id = payroll_calendar_id
        self.name = name
        self.calendar_type = calendar_type
        self.period_start_date = period_start_date
        if period_end_date is not None:
            self.period_end_date = period_end_date
        self.payment_date = payment_date
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc

    @property
    def payroll_calendar_id(self):
        """Gets the payroll_calendar_id of this PayRunCalendar.  # noqa: E501

        Xero unique identifier for the payroll calendar  # noqa: E501

        :return: The payroll_calendar_id of this PayRunCalendar.  # noqa: E501
        :rtype: str
        """
        return self._payroll_calendar_id

    @payroll_calendar_id.setter
    def payroll_calendar_id(self, payroll_calendar_id):
        """Sets the payroll_calendar_id of this PayRunCalendar.

        Xero unique identifier for the payroll calendar  # noqa: E501

        :param payroll_calendar_id: The payroll_calendar_id of this PayRunCalendar.  # noqa: E501
        :type: str
        """

        self._payroll_calendar_id = payroll_calendar_id

    @property
    def name(self):
        """Gets the name of this PayRunCalendar.  # noqa: E501

        Name of the calendar  # noqa: E501

        :return: The name of this PayRunCalendar.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this PayRunCalendar.

        Name of the calendar  # noqa: E501

        :param name: The name of this PayRunCalendar.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def calendar_type(self):
        """Gets the calendar_type of this PayRunCalendar.  # noqa: E501

        Type of the calendar  # noqa: E501

        :return: The calendar_type of this PayRunCalendar.  # noqa: E501
        :rtype: str
        """
        return self._calendar_type

    @calendar_type.setter
    def calendar_type(self, calendar_type):
        """Sets the calendar_type of this PayRunCalendar.

        Type of the calendar  # noqa: E501

        :param calendar_type: The calendar_type of this PayRunCalendar.  # noqa: E501
        :type: str
        """
        if calendar_type is None:
            raise ValueError(
                "Invalid value for `calendar_type`, must not be `None`"
            )  # noqa: E501
        allowed_values = [
            "Weekly",
            "Fortnightly",
            "FourWeekly",
            "Monthly",
            "Annual",
            "Quarterly",
            "None",
        ]  # noqa: E501

        if calendar_type:
            if calendar_type not in allowed_values:
                raise ValueError(
                    "Invalid value for `calendar_type` ({0}), must be one of {1}".format(  # noqa: E501
                        calendar_type, allowed_values
                    )
                )

        self._calendar_type = calendar_type

    @property
    def period_start_date(self):
        """Gets the period_start_date of this PayRunCalendar.  # noqa: E501

        Period start date of the calendar  # noqa: E501

        :return: The period_start_date of this PayRunCalendar.  # noqa: E501
        :rtype: date
        """
        return self._period_start_date

    @period_start_date.setter
    def period_start_date(self, period_start_date):
        """Sets the period_start_date of this PayRunCalendar.

        Period start date of the calendar  # noqa: E501

        :param period_start_date: The period_start_date of this PayRunCalendar.  # noqa: E501
        :type: date
        """
        if period_start_date is None:
            raise ValueError(
                "Invalid value for `period_start_date`, must not be `None`"
            )  # noqa: E501

        self._period_start_date = period_start_date

    @property
    def period_end_date(self):
        """Gets the period_end_date of this PayRunCalendar.  # noqa: E501

        Period end date of the calendar  # noqa: E501

        :return: The period_end_date of this PayRunCalendar.  # noqa: E501
        :rtype: date
        """
        return self._period_end_date

    @period_end_date.setter
    def period_end_date(self, period_end_date):
        """Sets the period_end_date of this PayRunCalendar.

        Period end date of the calendar  # noqa: E501

        :param period_end_date: The period_end_date of this PayRunCalendar.  # noqa: E501
        :type: date
        """

        self._period_end_date = period_end_date

    @property
    def payment_date(self):
        """Gets the payment_date of this PayRunCalendar.  # noqa: E501

        Payment date of the calendar  # noqa: E501

        :return: The payment_date of this PayRunCalendar.  # noqa: E501
        :rtype: date
        """
        return self._payment_date

    @payment_date.setter
    def payment_date(self, payment_date):
        """Sets the payment_date of this PayRunCalendar.

        Payment date of the calendar  # noqa: E501

        :param payment_date: The payment_date of this PayRunCalendar.  # noqa: E501
        :type: date
        """
        if payment_date is None:
            raise ValueError(
                "Invalid value for `payment_date`, must not be `None`"
            )  # noqa: E501

        self._payment_date = payment_date

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this PayRunCalendar.  # noqa: E501

        UTC timestamp of the last update to the pay run calendar  # noqa: E501

        :return: The updated_date_utc of this PayRunCalendar.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this PayRunCalendar.

        UTC timestamp of the last update to the pay run calendar  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this PayRunCalendar.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc



# coding: utf-8

"""
    Xero oAuth 2 identity service

    This specifing endpoints related to managing authentication tokens and identity for Xero API  # noqa: E501

    OpenAPI spec version: 2.0.4
    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


class OpenApiException(Exception):
    """The base exception class for all OpenAPIExceptions"""


class ApiException(OpenApiException):
    def __init__(self, status=None, reason=None, http_resp=None):
        self._status = status
        self._reason = reason
        self.http_resp = http_resp

    @property
    def status(self):
        return self._status or getattr(self.http_resp, "status", None)

    @property
    def reason(self):
        return self._reason or getattr(self.http_resp, "reason", None)

    @property
    def body(self):
        return getattr(self.http_resp, "data", None)

    @property
    def headers(self):
        return self.http_resp.getheaders() if self.http_resp else None

    @property
    def error_message(self):
        return "({0})\n" "Reason: {1}".format(self.status, self.reason)

    def __str__(self):
        """Custom error messages for exception"""
        error_message = self.error_message + "\n"
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


class ApiSSLException(ApiException):
    """Exception in case there is SSL related error"""


class OAuth2Error(OpenApiException):
    """The base exception class for all OAuth2 exception"""


# import all HTTP status exceptions
from .http_status_exceptions import *  # noqa


class AccessTokenExpiredError(OAuth2Error):
    """
    Exception for invalid access token
    """


class OAuth2TokenGetterError(OAuth2Error):
    """
    Exception for invalid oauth2 token getter function
    """


class OAuth2TokenSaverError(OAuth2Error):
    """
    Exception for invalid oauth2 token saver function
    """


class ApiTypeError(OpenApiException, TypeError):
    def __init__(self, msg, path_to_item=None, valid_classes=None, key_type=None):
        """Raises an exception for TypeErrors

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list): a list of keys an indices to get to the
                                 current_item
                                 None if unset
            valid_classes (tuple): the primitive classes that current item
                                   should be an instance of
                                   None if unset
            key_type (bool): False if our value is a value in a dict
                             True if it is a key in a dict
                             False if our item is an item in a list
                             None if unset
        """
        self.path_to_item = path_to_item
        self.valid_classes = valid_classes
        self.key_type = key_type
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiTypeError, self).__init__(full_msg)


class ApiValueError(OpenApiException, ValueError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list) the path to the exception in the
                received_data dict. None if unset
        """

        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiValueError, self).__init__(full_msg)


class ApiKeyError(OpenApiException, KeyError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiKeyError, self).__init__(full_msg)


def render_path(path_to_item):
    """Returns a string representation of a path"""
    result = ""
    for pth in path_to_item:
        if isinstance(pth, int):
            result += "[{0}]".format(pth)
        else:
            result += "['{0}']".format(pth)
    return result



# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class EarningsRate(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "earnings_rate_id": "str",
        "name": "str",
        "earnings_type": "str",
        "rate_type": "str",
        "type_of_units": "str",
        "current_record": "bool",
        "expense_account_id": "str",
        "rate_per_unit": "float",
        "multiple_of_ordinary_earnings_rate": "float",
        "fixed_amount": "float",
    }

    attribute_map = {
        "earnings_rate_id": "earningsRateID",
        "name": "name",
        "earnings_type": "earningsType",
        "rate_type": "rateType",
        "type_of_units": "typeOfUnits",
        "current_record": "currentRecord",
        "expense_account_id": "expenseAccountID",
        "rate_per_unit": "ratePerUnit",
        "multiple_of_ordinary_earnings_rate": "multipleOfOrdinaryEarningsRate",
        "fixed_amount": "fixedAmount",
    }

    def __init__(
        self,
        earnings_rate_id=None,
        name=None,
        earnings_type=None,
        rate_type=None,
        type_of_units=None,
        current_record=None,
        expense_account_id=None,
        rate_per_unit=None,
        multiple_of_ordinary_earnings_rate=None,
        fixed_amount=None,
    ):  # noqa: E501
        """EarningsRate - a model defined in OpenAPI"""  # noqa: E501

        self._earnings_rate_id = None
        self._name = None
        self._earnings_type = None
        self._rate_type = None
        self._type_of_units = None
        self._current_record = None
        self._expense_account_id = None
        self._rate_per_unit = None
        self._multiple_of_ordinary_earnings_rate = None
        self._fixed_amount = None
        self.discriminator = None

        if earnings_rate_id is not None:
            self.earnings_rate_id = earnings_rate_id
        self.name = name
        self.earnings_type = earnings_type
        self.rate_type = rate_type
        self.type_of_units = type_of_units
        if current_record is not None:
            self.current_record = current_record
        self.expense_account_id = expense_account_id
        if rate_per_unit is not None:
            self.rate_per_unit = rate_per_unit
        if multiple_of_ordinary_earnings_rate is not None:
            self.multiple_of_ordinary_earnings_rate = multiple_of_ordinary_earnings_rate
        if fixed_amount is not None:
            self.fixed_amount = fixed_amount

    @property
    def earnings_rate_id(self):
        """Gets the earnings_rate_id of this EarningsRate.  # noqa: E501

        Xero unique identifier for an earning rate  # noqa: E501

        :return: The earnings_rate_id of this EarningsRate.  # noqa: E501
        :rtype: str
        """
        return self._earnings_rate_id

    @earnings_rate_id.setter
    def earnings_rate_id(self, earnings_rate_id):
        """Sets the earnings_rate_id of this EarningsRate.

        Xero unique identifier for an earning rate  # noqa: E501

        :param earnings_rate_id: The earnings_rate_id of this EarningsRate.  # noqa: E501
        :type: str
        """

        self._earnings_rate_id = earnings_rate_id

    @property
    def name(self):
        """Gets the name of this EarningsRate.  # noqa: E501

        Name of the earning rate  # noqa: E501

        :return: The name of this EarningsRate.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this EarningsRate.

        Name of the earning rate  # noqa: E501

        :param name: The name of this EarningsRate.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def earnings_type(self):
        """Gets the earnings_type of this EarningsRate.  # noqa: E501

        Indicates how an employee will be paid when taking this type of earning  # noqa: E501

        :return: The earnings_type of this EarningsRate.  # noqa: E501
        :rtype: str
        """
        return self._earnings_type

    @earnings_type.setter
    def earnings_type(self, earnings_type):
        """Sets the earnings_type of this EarningsRate.

        Indicates how an employee will be paid when taking this type of earning  # noqa: E501

        :param earnings_type: The earnings_type of this EarningsRate.  # noqa: E501
        :type: str
        """
        if earnings_type is None:
            raise ValueError(
                "Invalid value for `earnings_type`, must not be `None`"
            )  # noqa: E501
        allowed_values = [
            "Allowance",
            "Backpay",
            "Bonus",
            "Commission",
            "DiscretionaryPayments",
            "HolidayPay",
            "LumpSum",
            "OtherEarnings",
            "OtherGrossEarnings",
            "OvertimeEarnings",
            "RegularEarnings",
            "SalarySacrificeForKiwiSaver",
            "Tips(Direct)",
            "Tips(Non-Direct)",
            "WithholdingIncome",
            "None",
        ]  # noqa: E501

        if earnings_type:
            if earnings_type not in allowed_values:
                raise ValueError(
                    "Invalid value for `earnings_type` ({0}), must be one of {1}".format(  # noqa: E501
                        earnings_type, allowed_values
                    )
                )

        self._earnings_type = earnings_type

    @property
    def rate_type(self):
        """Gets the rate_type of this EarningsRate.  # noqa: E501

        Indicates the type of the earning rate  # noqa: E501

        :return: The rate_type of this EarningsRate.  # noqa: E501
        :rtype: str
        """
        return self._rate_type

    @rate_type.setter
    def rate_type(self, rate_type):
        """Sets the rate_type of this EarningsRate.

        Indicates the type of the earning rate  # noqa: E501

        :param rate_type: The rate_type of this EarningsRate.  # noqa: E501
        :type: str
        """
        if rate_type is None:
            raise ValueError(
                "Invalid value for `rate_type`, must not be `None`"
            )  # noqa: E501
        allowed_values = [
            "RatePerUnit",
            "MultipleOfOrdinaryEarningsRate",
            "FixedAmount",
            "None",
        ]  # noqa: E501

        if rate_type:
            if rate_type not in allowed_values:
                raise ValueError(
                    "Invalid value for `rate_type` ({0}), must be one of {1}".format(  # noqa: E501
                        rate_type, allowed_values
                    )
                )

        self._rate_type = rate_type

    @property
    def type_of_units(self):
        """Gets the type_of_units of this EarningsRate.  # noqa: E501

        The type of units used to record earnings  # noqa: E501

        :return: The type_of_units of this EarningsRate.  # noqa: E501
        :rtype: str
        """
        return self._type_of_units

    @type_of_units.setter
    def type_of_units(self, type_of_units):
        """Sets the type_of_units of this EarningsRate.

        The type of units used to record earnings  # noqa: E501

        :param type_of_units: The type_of_units of this EarningsRate.  # noqa: E501
        :type: str
        """
        if type_of_units is None:
            raise ValueError(
                "Invalid value for `type_of_units`, must not be `None`"
            )  # noqa: E501

        self._type_of_units = type_of_units

    @property
    def current_record(self):
        """Gets the current_record of this EarningsRate.  # noqa: E501

        Indicates whether an earning type is active  # noqa: E501

        :return: The current_record of this EarningsRate.  # noqa: E501
        :rtype: bool
        """
        return self._current_record

    @current_record.setter
    def current_record(self, current_record):
        """Sets the current_record of this EarningsRate.

        Indicates whether an earning type is active  # noqa: E501

        :param current_record: The current_record of this EarningsRate.  # noqa: E501
        :type: bool
        """

        self._current_record = current_record

    @property
    def expense_account_id(self):
        """Gets the expense_account_id of this EarningsRate.  # noqa: E501

        The account that will be used for the earnings rate  # noqa: E501

        :return: The expense_account_id of this EarningsRate.  # noqa: E501
        :rtype: str
        """
        return self._expense_account_id

    @expense_account_id.setter
    def expense_account_id(self, expense_account_id):
        """Sets the expense_account_id of this EarningsRate.

        The account that will be used for the earnings rate  # noqa: E501

        :param expense_account_id: The expense_account_id of this EarningsRate.  # noqa: E501
        :type: str
        """
        if expense_account_id is None:
            raise ValueError(
                "Invalid value for `expense_account_id`, must not be `None`"
            )  # noqa: E501

        self._expense_account_id = expense_account_id

    @property
    def rate_per_unit(self):
        """Gets the rate_per_unit of this EarningsRate.  # noqa: E501

        Default rate per unit (optional). Only applicable if RateType is RatePerUnit  # noqa: E501

        :return: The rate_per_unit of this EarningsRate.  # noqa: E501
        :rtype: float
        """
        return self._rate_per_unit

    @rate_per_unit.setter
    def rate_per_unit(self, rate_per_unit):
        """Sets the rate_per_unit of this EarningsRate.

        Default rate per unit (optional). Only applicable if RateType is RatePerUnit  # noqa: E501

        :param rate_per_unit: The rate_per_unit of this EarningsRate.  # noqa: E501
        :type: float
        """

        self._rate_per_unit = rate_per_unit

    @property
    def multiple_of_ordinary_earnings_rate(self):
        """Gets the multiple_of_ordinary_earnings_rate of this EarningsRate.  # noqa: E501

        This is the multiplier used to calculate the rate per unit, based on the employee’s ordinary earnings rate. For example, for time and a half enter 1.5. Only applicable if RateType is MultipleOfOrdinaryEarningsRate  # noqa: E501

        :return: The multiple_of_ordinary_earnings_rate of this EarningsRate.  # noqa: E501
        :rtype: float
        """
        return self._multiple_of_ordinary_earnings_rate

    @multiple_of_ordinary_earnings_rate.setter
    def multiple_of_ordinary_earnings_rate(self, multiple_of_ordinary_earnings_rate):
        """Sets the multiple_of_ordinary_earnings_rate of this EarningsRate.

        This is the multiplier used to calculate the rate per unit, based on the employee’s ordinary earnings rate. For example, for time and a half enter 1.5. Only applicable if RateType is MultipleOfOrdinaryEarningsRate  # noqa: E501

        :param multiple_of_ordinary_earnings_rate: The multiple_of_ordinary_earnings_rate of this EarningsRate.  # noqa: E501
        :type: float
        """

        self._multiple_of_ordinary_earnings_rate = multiple_of_ordinary_earnings_rate

    @property
    def fixed_amount(self):
        """Gets the fixed_amount of this EarningsRate.  # noqa: E501

        Optional Fixed Rate Amount. Applicable for FixedAmount Rate  # noqa: E501

        :return: The fixed_amount of this EarningsRate.  # noqa: E501
        :rtype: float
        """
        return self._fixed_amount

    @fixed_amount.setter
    def fixed_amount(self, fixed_amount):
        """Sets the fixed_amount of this EarningsRate.

        Optional Fixed Rate Amount. Applicable for FixedAmount Rate  # noqa: E501

        :param fixed_amount: The fixed_amount of this EarningsRate.  # noqa: E501
        :type: float
        """

        self._fixed_amount = fixed_amount



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class PayItems(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"pay_items": "PayItem"}

    attribute_map = {"pay_items": "PayItems"}

    def __init__(self, pay_items=None):  # noqa: E501
        """PayItems - a model defined in OpenAPI"""  # noqa: E501

        self._pay_items = None
        self.discriminator = None

        if pay_items is not None:
            self.pay_items = pay_items

    @property
    def pay_items(self):
        """Gets the pay_items of this PayItems.  # noqa: E501


        :return: The pay_items of this PayItems.  # noqa: E501
        :rtype: PayItem
        """
        return self._pay_items

    @pay_items.setter
    def pay_items(self, pay_items):
        """Sets the pay_items of this PayItems.


        :param pay_items: The pay_items of this PayItems.  # noqa: E501
        :type: PayItem
        """

        self._pay_items = pay_items



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Allocation(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "invoice": "Invoice",
        "overpayment": "Overpayment",
        "prepayment": "Prepayment",
        "credit_note": "CreditNote",
        "amount": "float",
        "date": "date[ms-format]",
        "status_attribute_string": "str",
        "validation_errors": "list[ValidationError]",
    }

    attribute_map = {
        "invoice": "Invoice",
        "overpayment": "Overpayment",
        "prepayment": "Prepayment",
        "credit_note": "CreditNote",
        "amount": "Amount",
        "date": "Date",
        "status_attribute_string": "StatusAttributeString",
        "validation_errors": "ValidationErrors",
    }

    def __init__(
        self,
        invoice=None,
        overpayment=None,
        prepayment=None,
        credit_note=None,
        amount=None,
        date=None,
        status_attribute_string=None,
        validation_errors=None,
    ):  # noqa: E501
        """Allocation - a model defined in OpenAPI"""  # noqa: E501

        self._invoice = None
        self._overpayment = None
        self._prepayment = None
        self._credit_note = None
        self._amount = None
        self._date = None
        self._status_attribute_string = None
        self._validation_errors = None
        self.discriminator = None

        self.invoice = invoice
        if overpayment is not None:
            self.overpayment = overpayment
        if prepayment is not None:
            self.prepayment = prepayment
        if credit_note is not None:
            self.credit_note = credit_note
        self.amount = amount
        self.date = date
        if status_attribute_string is not None:
            self.status_attribute_string = status_attribute_string
        if validation_errors is not None:
            self.validation_errors = validation_errors

    @property
    def invoice(self):
        """Gets the invoice of this Allocation.  # noqa: E501


        :return: The invoice of this Allocation.  # noqa: E501
        :rtype: Invoice
        """
        return self._invoice

    @invoice.setter
    def invoice(self, invoice):
        """Sets the invoice of this Allocation.


        :param invoice: The invoice of this Allocation.  # noqa: E501
        :type: Invoice
        """
        if invoice is None:
            raise ValueError(
                "Invalid value for `invoice`, must not be `None`"
            )  # noqa: E501

        self._invoice = invoice

    @property
    def overpayment(self):
        """Gets the overpayment of this Allocation.  # noqa: E501


        :return: The overpayment of this Allocation.  # noqa: E501
        :rtype: Overpayment
        """
        return self._overpayment

    @overpayment.setter
    def overpayment(self, overpayment):
        """Sets the overpayment of this Allocation.


        :param overpayment: The overpayment of this Allocation.  # noqa: E501
        :type: Overpayment
        """

        self._overpayment = overpayment

    @property
    def prepayment(self):
        """Gets the prepayment of this Allocation.  # noqa: E501


        :return: The prepayment of this Allocation.  # noqa: E501
        :rtype: Prepayment
        """
        return self._prepayment

    @prepayment.setter
    def prepayment(self, prepayment):
        """Sets the prepayment of this Allocation.


        :param prepayment: The prepayment of this Allocation.  # noqa: E501
        :type: Prepayment
        """

        self._prepayment = prepayment

    @property
    def credit_note(self):
        """Gets the credit_note of this Allocation.  # noqa: E501


        :return: The credit_note of this Allocation.  # noqa: E501
        :rtype: CreditNote
        """
        return self._credit_note

    @credit_note.setter
    def credit_note(self, credit_note):
        """Sets the credit_note of this Allocation.


        :param credit_note: The credit_note of this Allocation.  # noqa: E501
        :type: CreditNote
        """

        self._credit_note = credit_note

    @property
    def amount(self):
        """Gets the amount of this Allocation.  # noqa: E501

        the amount being applied to the invoice  # noqa: E501

        :return: The amount of this Allocation.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this Allocation.

        the amount being applied to the invoice  # noqa: E501

        :param amount: The amount of this Allocation.  # noqa: E501
        :type: float
        """
        if amount is None:
            raise ValueError(
                "Invalid value for `amount`, must not be `None`"
            )  # noqa: E501

        self._amount = amount

    @property
    def date(self):
        """Gets the date of this Allocation.  # noqa: E501

        the date the allocation is applied YYYY-MM-DD.  # noqa: E501

        :return: The date of this Allocation.  # noqa: E501
        :rtype: date
        """
        return self._date

    @date.setter
    def date(self, date):
        """Sets the date of this Allocation.

        the date the allocation is applied YYYY-MM-DD.  # noqa: E501

        :param date: The date of this Allocation.  # noqa: E501
        :type: date
        """
        if date is None:
            raise ValueError(
                "Invalid value for `date`, must not be `None`"
            )  # noqa: E501

        self._date = date

    @property
    def status_attribute_string(self):
        """Gets the status_attribute_string of this Allocation.  # noqa: E501

        A string to indicate if a invoice status  # noqa: E501

        :return: The status_attribute_string of this Allocation.  # noqa: E501
        :rtype: str
        """
        return self._status_attribute_string

    @status_attribute_string.setter
    def status_attribute_string(self, status_attribute_string):
        """Sets the status_attribute_string of this Allocation.

        A string to indicate if a invoice status  # noqa: E501

        :param status_attribute_string: The status_attribute_string of this Allocation.  # noqa: E501
        :type: str
        """

        self._status_attribute_string = status_attribute_string

    @property
    def validation_errors(self):
        """Gets the validation_errors of this Allocation.  # noqa: E501

        Displays array of validation error messages from the API  # noqa: E501

        :return: The validation_errors of this Allocation.  # noqa: E501
        :rtype: list[ValidationError]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this Allocation.

        Displays array of validation error messages from the API  # noqa: E501

        :param validation_errors: The validation_errors of this Allocation.  # noqa: E501
        :type: list[ValidationError]
        """

        self._validation_errors = validation_errors
